/** <!------------------------------------------------------------------------>
 * @file helloworld.c
 *
 * (c) 2000 - 2006 by Markus Grunwald
 <!------------------------------------------------------------------------> */

// --- includes --- 
#include "hardware.h"
#include "lcd.h"
#include <ctype.h>
#include <io.h>
#include <limits.h>
#include <msp430x14x.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

// --- defines ---

int nDummy=42; // To get rid of gdb errors

/** <!-------------------------------------------------------------------------->
 * Redirect stdout to LCD
 * 
 * @param nChar (i) Character to print
 * 
 * @return number of printed characters
 <!--------------------------------------------------------------------------> */
int putchar(int c)
{
    if( isprint( c ) )
    {
        lcdPutc(c);
    }
    return 1;
}

/**
Unspecific delay function (busy wait type).
this small assembler bit does not need any C init etc. so
we can use the attribute "naked"
*/
__attribute__ ((naked)) void delay(unsigned int n)
{
    //first arg comes in register R15
    //the loop uses 3 cycles per round
    //the eight cycles come from the call overhead and ret
    //
    //delay_time = (1/MCLK)*(8+(3*n))
    asm("lcdloop: dec %0\n  jnz lcdloop\n ret" :: "r" (n));
}

/** <!-------------------------------------------------------------------------->
 * Key was turned left
 <!---------------------------------------------------------------------------> */
void dimBlueLED( void )
{
    if( TACCR1>0 )
    {
        --TACCR1;  // Dim blue LED
    }
} // END dimBlueLED

/** <!-------------------------------------------------------------------------->
 * Key was turned right
 <!---------------------------------------------------------------------------> */
void incBlueLED( void )
{
    if( TACCR1<511  )
    {
        ++TACCR1;  // Make blue LED brighter
    }
} // END incBlueLED

/** <!-------------------------------------------------------------------------->
 * Key was pushed
 <!---------------------------------------------------------------------------> */
void toggleBlueLED( void )
{
    // Toggle blue LED
    if( TACCR1 )
    {
        TACCR1=0;
    }
    else 
    {
        TACCR1=511;
    }

    // The rotary-push button does not allways stay low, when it is pressed.
    // So we skip contact de-bounce and just wait some time until the contact is settled.
    delay( DELAY1MS * 100 );
} // END toggleBlueLED




// Keyboard interrupt
interrupt (PORT1_VECTOR) Port1Int(void)
{
    int nDirection=0;      // 0 == left
    if( P1IN & ROTKEY_B  )
    {
        nDirection=1;
    }

    if( P1IFG & ROTKEY_A  )
    {
        // Turnkey, line A
        if( nDirection )
        {
            incBlueLED();
        }
        else
        {
            dimBlueLED();
        }
    }
    else if( P1IFG & ROTKEY_BTN )
    {
        toggleBlueLED();
    }

    P1IFG =0x00;
}

/** <!-------------------------------------------------------------------------->
 * Interrupt vector fort timer B0. I hope, this will handle the rtc, sometimes...
 <!---------------------------------------------------------------------------> */
wakeup interrupt (TIMERB0_VECTOR) TimerB0Int( void )
{
    TBCCR0 += TB_ONESEC;        // Next Interrupt in one second
/*     P1OUT ^= LED_YELLOW; */

} // END TimerB0Int

/** <!-------------------------------------------------------------------------->
 * Initialises the controller
 <!---------------------------------------------------------------------------> */
void initialise( void )
{
    WDTCTL = WDTCTL_INIT;               //Init watchdog timer

    DCOCTL = DCOCTL_INIT;               // Init DCO Controll register
    BCSCTL1= BCSCTL1_INIT;              // Init Basic Clock System Control Register 1

    P1OUT  = P1OUT_INIT;                //Init output data of port1
    P5OUT  = P5OUT_INIT;                //Init output data of port2

    P1SEL  = P1SEL_INIT;                //Select port or module -function on port1
    P5SEL  = P5SEL_INIT;                //Select port or module -function on port2

    P1DIR  = P1DIR_INIT;                //Init port direction register of port1
    P5DIR  = P5DIR_INIT;                //Init port direction register of port2

    P1IES  = P1IES_INIT;                //init port interrupts
    P2IES  = P2IES_INIT;
    P1IE   = P1IE_INIT;
    P2IE   = P2IE_INIT;

    TACTL  = TACTL_INIT;                // setup timer A (still stopped)
    TACCR0 = 512-1;                     // PWM Period.  Set OUT, when counted to TACCR0
    TACCTL1= OUTMOD_7;                  // CCR1 reset/set
    TACCR1 = 0;                         // CCR1 PWM duty cycle. Reset, when counted to TACCR1

    TBCTL  = TBCTL_INIT;                // setup timer B
    TBCCTL0= TBCCTL0_INIT;              // setup capture/compare unit 0
    TBCCR0 = TB_ONESEC;                 // One interrupt each second
    TBCTL |= MC_2;                      // MC_2: Continous mode

    eint();                             //enable interrupts

    TACTL |= MC_1;                      // Start Timer_A in up mode
    TACCR1 = 0;                         // CCR1 PWM duty cycle. Reset, when counted to TACCR1


    lcdOn();                            //reset sequence
    lcdInit();                          //setup LCD modes
} // END initialise


int main( int nArgc, char* aaszArgv[] )
{
    initialise();

    lcdInstr(LCD_LINE1);
    puts("1st Line");            //say hello

    for (;;)
    {
        P1OUT ^= LED_YELLOW;
        LPM0;
    }
    return 0;
}


// Local Variables:
// gdb-command-name: msp430-gdb
// End:
// EOF
